@@ -221,7 +221,7 @@ func TestMostTypes(t *testing.T) {
 		return
 	}
 
-	// Go through each field and check to see it got copied properly
+	// go through each field and check to see it got copied properly
 	if cpy.String != test.String {
 		t.Errorf("String: got %v; want %v", cpy.String, test.String)
 	}
@@ -579,7 +579,7 @@ Interfaces:
 // not meant to be exhaustive
 // nolint
 func TestComplexSlices(t *testing.T) {
-	orig3Int := [][][]int{[][]int{[]int{1, 2, 3}, []int{11, 22, 33}}, [][]int{[]int{7, 8, 9}, []int{66, 77, 88, 99}}}
+	orig3Int := [][][]int{{{1, 2, 3}, {11, 22, 33}}, {{7, 8, 9}, {66, 77, 88, 99}}}
 	cpyI := Copy(orig3Int).([][][]int)
 	if (*reflect.SliceHeader)(unsafe.Pointer(&orig3Int)).Data == (*reflect.SliceHeader)(unsafe.Pointer(&cpyI)).Data {
 		t.Error("[][][]int: address of copy was the same as original; they should be different")
@@ -609,7 +609,7 @@ func TestComplexSlices(t *testing.T) {
 	}
 
 sliceMap:
-	slMap := []map[int]string{map[int]string{0: "a", 1: "b"}, map[int]string{10: "k", 11: "l", 12: "m"}}
+	slMap := []map[int]string{{0: "a", 1: "b"}, {10: "k", 11: "l", 12: "m"}}
 	cpyM := Copy(slMap).([]map[int]string)
 	if (*reflect.SliceHeader)(unsafe.Pointer(&slMap)).Data == (*reflect.SliceHeader)(unsafe.Pointer(&cpyM)).Data {
 		t.Error("[]map[int]string: address of copy was the same as original; they should be different")
@@ -659,11 +659,11 @@ var AStruct = A{
 	UintSl: []uint{0, 1, 2, 3},
 	Map:    map[string]int{"a": 1, "b": 2},
 	MapB: map[string]*B{
-		"hi":  &B{Vals: []string{"hello", "bonjour"}},
-		"bye": &B{Vals: []string{"good-bye", "au revoir"}},
+		"hi":  {Vals: []string{"hello", "bonjour"}},
+		"bye": {Vals: []string{"good-bye", "au revoir"}},
 	},
 	SliceB: []B{
-		B{Vals: []string{"Ciao", "Aloha"}},
+		{Vals: []string{"Ciao", "Aloha"}},
 	},
 	B: B{Vals: []string{"42"}},
 	T: time.Now(),
@@ -948,13 +948,13 @@ func TestIssue9(t *testing.T) {
 
 	testB := Biz{
 		Epsilon: map[int]*Bar{
-			0: &Bar{},
-			1: &Bar{
+			0: {},
+			1: {
 				Beta:  "don't panic",
 				Gamma: 42,
 				Delta: nil,
 			},
-			2: &Bar{
+			2: {
 				Beta:  "sudo make me a sandwich.",
 				Gamma: 11,
 				Delta: &Foo{
@@ -1010,10 +1010,10 @@ func TestIssue9(t *testing.T) {
 
 	// test that map keys are deep copied
 	testC := map[*Foo][]string{
-		&Foo{Alpha: "Henry Dorsett Case"}: []string{
+		{Alpha: "Henry Dorsett Case"}: {
 			"Cutter",
 		},
-		&Foo{Alpha: "Molly Millions"}: []string{
+		{Alpha: "Molly Millions"}: {
 			"Rose Kolodny",
 			"Cat Mother",
 			"Steppin' Razor",
@@ -1056,8 +1056,8 @@ func TestIssue9(t *testing.T) {
 	}
 
 	testD := map[Bizz]string{
-		Bizz{&Foo{"Neuromancer"}}: "Rio",
-		Bizz{&Foo{"Wintermute"}}:  "Berne",
+		{&Foo{"Neuromancer"}}: "Rio",
+		{&Foo{"Wintermute"}}:  "Berne",
 	}
 	copyD := Copy(testD).(map[Bizz]string)
 	if len(copyD) != len(testD) {
